local randomStartingRotations = {90, 270, 180, 0}
local standardObjRotation = 90 -- magicavoxel .obj files are always rotated 90 degs

local GroundTile = ECS.Component(function(param)
    -- Represents a model that composes a ground tile. Used to render the game world
    assert(param.model_obj, "[Component][GroundTile] Attempt to create null ground tile model!")
    assert(param.model_texture, "[Component][GroundTile] Attempt to create null ground tile texture!")
    
    local xRot, yRot, zRot = standardObjRotation, 0, randomStartingRotations[math.random( #randomStartingRotations )]
    local xScale, yScale, zScale = 1, 1, 1
    local xPos, yPos, zPos = 0, 0, 0

    if param.position then
        xPos = param.position.x or xPos
        yPos = param.position.y or yPos
        zPos = param.position.z or zPos
    end

    if param.rotation then
        xRot = param.rotation.x or xRot
        yRot = param.rotation.y or yRot
        zRot = param.rotation.z or zRot
        if xRot == 0 then xRot = standardObjRotation end
    end

    if param.scale then
        xScale = param.scale.x or xScale
        yScale = param.scale.y or yScale
        zScale = param.scale.z or zScale
    end

    local loaded_model = G3D.newModel(param.model_obj, param.model_texture,
        {xPos, yPos, zPos},                                                 -- position
        {xRot*(math.pi/180), yRot*(math.pi/180), zRot*(math.pi/180)},       -- rotation
        {xScale, yScale, zScale}                                               -- scale
    )
    loaded_model:makeNormals() -- calculate normals required for lighting

    return {
        model = loaded_model,
        position = {
            x = param.x or 0,
            y = param.y or 0,
            z = param.z or 0
        },
        name = param.name or "Unnamed tile"
    }
end)
return GroundTile
