local gameScene = {}

-- ECS game world (ECS)
gCurrentGameWorld = ECS.World({disableAutoUpdate = true})

-- ECS components
local GroundBlock = require("bin.components.level_geometry.GroundBlock")

-- ECS systems
local GroundDrawSystem = require("bin.systems.GroundDrawSystem")
local CameraControlSystem = require("bin.systems.CameraControlSystem")

-- resources
local block_mesh = G3D.loadObj("gfx/tiles/grass/grass-block.obj")
local block_texture = love.graphics.newImage("gfx/tiles/grass/grass-block.png")

function gameScene.load()
    -- attach systems
    love.graphics.setMeshCullMode("back")
    gCurrentGameWorld:AddSystem(CameraControlSystem)
    gCurrentGameWorld:AddSystem(GroundDrawSystem)

    -- put some objects in scenery
    gCurrentGameWorld:Entity(
        GroundBlock({
            model_obj = block_mesh,
            model_texture = block_texture,
            position = {
                x = 0,
                y = 0,
                z = 0
            }
        })
    )

    gCurrentGameWorld:Entity(
        GroundBlock({
            model_obj = block_mesh,
            model_texture = block_texture,
            position = {
                x = 0,
                y = 4,
                z = 0
            }
        })
    )

    gCurrentGameWorld:Entity(
        GroundBlock({
            model_obj = block_mesh,
            model_texture = block_texture,
            position = {
                x = 0,
                y = 8,
                z = 0
            }
        })
    )

    gCurrentGameWorld:Entity(
        GroundBlock({
            model_obj = block_mesh,
            model_texture = block_texture,
            position = {
                x = 0,
                y = 12,
                z = 0
            }
        })
    )

    -- add a special block to render where light source (supposedly) is
    gCurrentGameWorld:Entity(
        GroundBlock({
            model_obj = "gfx/tiles/grass/lightmarker.obj",
            model_texture = "gfx/tiles/grass/lightmarker.png",
            position = {
                x = 6,
                y = 6,
                z = 6
            }
        })
    )

    love.graphics.setBackgroundColor(0, 0, 48/255, 1)
    G3D.camera.lookInDirection(
        8,
        6,
        8)
end

function gameScene.update(dt)
    --update entire gameworld
    gCurrentGameWorld:Update("process", love.timer.getTime())
    gCurrentGameWorld:Update("transform", love.timer.getTime())
end

function gameScene.draw()
    gCurrentGameWorld:Update("render", love.timer.getTime())
    love.graphics.print("Blue block is light source", 0, 52)
    love.graphics.print("WASD to move, QE to rotate, [] to pitch, ESC quits", 0, 64)
end

return gameScene
