local CameraControlSystem = ECS.System("transform")

function CameraControlSystem:Initialize(config)
    print("[CameraControlSystem] Initialized system! 'transform'")
    self.cameraMoved = true
    self.moveSpeed = 2

    self.direction = 180*(math.pi/180)
    self.pitch = -54*(math.pi/180)
end

function CameraControlSystem:Update(time)
    local up    = love.keyboard.isScancodeDown('w', 'up') and (self.moveSpeed*time.DeltaFixed) or 0
    local down  = love.keyboard.isScancodeDown('s', 'down') and (self.moveSpeed*time.DeltaFixed) or 0
    local left  = love.keyboard.isScancodeDown('a', 'left') and (self.moveSpeed*time.DeltaFixed) or 0
    local right = love.keyboard.isScancodeDown('d', 'right') and (self.moveSpeed*time.DeltaFixed) or 0
    local e_key = love.keyboard.isScancodeDown('e') and (time.DeltaFixed) or 0
    local q_key = love.keyboard.isScancodeDown('q') and (time.DeltaFixed) or 0
    local spacebar_key = love.keyboard.isScancodeDown('space') and (1*time.DeltaFixed) or 0
    local lshift = love.keyboard.isScancodeDown('lshift') and (1*time.DeltaFixed) or 0
    local zoom_plus_key = love.keyboard.isScancodeDown('[') and (1*time.DeltaFixed) or 0
    local zoom_minus_key = love.keyboard.isScancodeDown(']') and (1*time.DeltaFixed) or 0
    local escape_key = love.keyboard.isScancodeDown('escape') and (1*time.DeltaFixed) or 0

    -- camera
    local camera = G3D.camera
    local moveX, moveY = 0, 0
    
    if up ~= 0 then moveX = moveX + 1 end
    if left ~= 0 then moveY = moveY + 1 end
    if down ~= 0 then moveX = moveX - 1 end
    if right ~= 0 then moveY = moveY - 1 end
    if spacebar_key ~= 0 then
        camera.position[3] = camera.position[3] + spacebar_key
        self.cameraMoved = true
    end
    if lshift ~= 0 then
        camera.position[3] = camera.position[3] - lshift
        self.cameraMoved = true
    end

    if e_key ~= 0 then 
        self.direction = self.direction + e_key
        self.cameraMoved = true
    end
    if q_key ~= 0 then
        self.direction = self.direction - q_key
        self.cameraMoved = true
    end
    if zoom_plus_key ~= 0 then
        self.pitch = self.pitch + zoom_plus_key
        self.cameraMoved = true
    end
    if zoom_minus_key ~= 0 then
        self.pitch = self.pitch - zoom_minus_key
        self.cameraMoved = true
    end

    if escape_key ~= 0 then
        love.event.quit(0)
    end
    
    -- do some trigonometry on the inputs to make movement relative to camera's direction
    -- also to make the player not move faster in diagonal directions
    if moveX ~= 0 or moveY ~= 0 then
        local angle = math.atan2(moveY, moveX)
        camera.position[1] = camera.position[1] + math.cos(self.direction + angle) * self.moveSpeed * time.DeltaFixed
        camera.position[2] = camera.position[2] + math.sin(self.direction + angle) * self.moveSpeed * time.DeltaFixed
        self.cameraMoved = true
    end

    -- update the camera's in the shader
    -- only if the camera moved, for a slight performance benefit
    if self.cameraMoved then
        camera.lookInDirection(nil,nil,nil, self.direction, self.pitch)
        self.cameraMoved = false
    end
end

return CameraControlSystem
