local GroundDrawSystem = ECS.System("render", 5)

-- components
local GroundTile = require("bin.components.level_geometry.GroundBlock")

-- shaders
local testShader = love.graphics.newShader(G3D.shaderpath, "test.frag")

function GroundDrawSystem:Initialize(config)
    print("[GroundDrawSystem] Initialized system! 'render'")
    
    -- initialize shader variables
    testShader:send('LightWorldPos', {4, 4, 4})
    testShader:send('LightColor', {1.0, 1.0, 1.0})

    self.queryGroundTiles = ECS.Query.All(GroundTile).Build()
end

function GroundDrawSystem:Update(time)
    local groundTile = GroundTile

    love.graphics.setColor(1, 1, 1)
    -- testShader:send('lightPos', {G3D.camera.position[1], G3D.camera.position[2], G3D.camera.position[2]})

    local tiles = self:Result(self.queryGroundTiles):ToArray()
    local tileSize = #tiles

    for i, tile in ipairs(tiles) do
        tile[groundTile].model:draw(testShader)
    end

    love.graphics.print("[CameraSystem] Camera pos: X: "..G3D.camera.position[1].." Y: "..G3D.camera.position[2].." Z: "..G3D.camera.position[3], 0, 0)
    love.graphics.print("[GroundDrawSystem] Tiles registered: "..tileSize, 0, 16)
    local stats = love.graphics.getStats()
    love.graphics.print("FPS: "..love.timer.getFPS().." DrawCalls: "..stats.drawcalls.." Batched:"..stats.drawcallsbatched, 0, 32)

end

return GroundDrawSystem
