uniform vec3 LightColor;

varying vec4 worldPosition;
varying vec3 lightPos;
varying vec3 transformedVertexNormal;

// related stuff: https://learnopengl.com/code_viewer_gh.php?code=src/2.lighting/2.1.basic_lighting_diffuse/2.1.basic_lighting.fs
// https://learnopengl.com/code_viewer_gh.php?code=src/2.lighting/2.1.basic_lighting_diffuse/basic_lighting_diffuse.cpp
// https://learnopengl.com/Lighting/Basic-Lighting
// https://github.com/groverburger/g3d/wiki/Creating-custom-g3d-shaders
// https://blogs.love2d.org/content/beginners-guide-shaders

vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords) {
    vec4 texcolor = Texel(texture, texture_coords);
    
    // ambient lighting
    float ambientStr = 0.8;
    vec3 ambientColor = ambientStr * LightColor;
    
    // diffuse lighting
    vec3 vertexPos = worldPosition.xyz;
    vec3 normal = normalize(transformedVertexNormal);
    vec3 lightDir = normalize(lightPos - vertexPos);
    float diff = max(dot(normal, lightDir), 0.0);
    vec3 diffuseColor = diff * LightColor;

    // color
    vec4 cur_color = (texcolor * color);

    // final composition
    vec4 finalAmbient = vec4(ambientColor, 1.0);
    vec4 finalDiffuse = vec4(diffuseColor, 1.0);
    vec4 result = (finalAmbient + finalDiffuse) * cur_color;


    // get rid of transparent pixels
    if (texcolor.a == 0.0) {
        discard;
    }
    
    return result;
}